# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

from slackoff import slack


def describe_signout():
    def it_indicates_success(expect):
        expect(slack.signout("Foobar")) == False



import log
from datafiles import datafile, field


@datafile
class Workspace:
    name: str
    active: bool = True
    uses: int = 0


@datafile("~/Library/Preferences/slackoff.yml")
class Settings:
    workspaces: list[Workspace] = field(default_factory=list)

    def activate(self, name: str):
        log.debug(f"Marking workspace active: {name}")
        workspace = self._update(name, True)
        workspace.uses += 1

    def deactivate(self, name: str):
        log.debug(f"Marking workspace inactive: {name}")
        self._update(name, False)

    def _update(self, name: str, active: bool):
        for workspace in self.workspaces:
            if workspace.name == name:
                workspace.active = active
                return workspace

        workspace = Workspace(name, active)
        self.workspaces.append(workspace)
        return workspace


settings = Settings()



from pathlib import Path

import applescript
import log


def call(path: Path, signature: str, *, show_error: bool = True) -> bool:
    functions = path.read_text("utf-8")

    log.debug(f"Calling AppleScript: {signature}")
    result = applescript.run(functions + "\n\n" + signature)

    log.debug(f"AppleScript code: {result.code}")
    if result.out:
        log.debug(f"AppleScript output: {result.out}")
    if result.err:
        log.debug(f"AppleScript error: {result.err}")
        if show_error:
            message = result.err.split("error:")[-1].strip()
            log.error(message)

    return result.code == 0 and "missing value" not in result.out



import sys

import click
import log
import pync

from . import __version__, browser, slack
from .config import settings


@click.command(help="Automatically sign out/in of a Slack workspace.")
@click.argument("workspace", nargs=-1)
@click.option(
    "-i", "--signin", is_flag=True, default=False, help="Only attempt to sign in."
)
@click.option(
    "-o", "--signout", is_flag=True, default=False, help="Only attempt to sign out."
)
@click.option(
    "--debug", is_flag=True, default=False, help="Show verbose logging output."
)
@click.version_option(__version__)
def main(workspace: str, signin: bool, signout: bool, debug: bool):
    log.init(debug=debug, format="%(levelname)s: %(message)s")

    workspace = get_workspace(workspace)

    if not (signin or signout) and not slack.activate():
        sys.exit(1)

    if signin:
        attempt_signin(workspace)
        sys.exit(0)

    if signout:
        attempt_signout(workspace)
        sys.exit(0)

    if not attempt_signout(workspace):
        click.echo(f"Signing in to {workspace}")
        attempt_signin(workspace)


def get_workspace(workspace: str) -> str:
    workspace = " ".join(workspace)

    if not workspace:
        if settings.workspaces:
            workspace = settings.workspaces[0].name
        else:
            workspace = click.prompt("Slack workspace")

    log.debug(f"Modifying workspace: {workspace}")
    return workspace


def attempt_signin(workspace) -> bool:
    if not slack.signin(workspace):
        message = f"Click 'Open' to sign in to {workspace}"
        pync.notify(message, title="Slackoff")
        click.echo(message)
        click.pause()
        browser.close()

    settings.activate(workspace)
    return True


def attempt_signout(workspace) -> bool:
    if slack.signout(workspace):
        click.echo(f"Signed out of {workspace}")
        settings.deactivate(workspace)
        return True

    click.echo(f"Currently signed out of {workspace}")
    return False


if __name__ == "__main__":  # pragma: no cover
    main()  # pylint: disable=no-value-for-parameter



"""Unit tests configuration file."""

import datafiles
import log


def pytest_configure(config):
    """Disable verbose output when running tests."""
    log.init(debug=True)

    terminal = config.pluginmanager.getplugin("terminal")
    terminal.TerminalReporter.showfspath = False


def pytest_runtest_setup(item):  # pylint: disable=unused-argument
    """Disable file storage during unit tests."""
    datafiles.settings.HOOKS_ENABLED = False



# pylint: disable=redefined-outer-name,unused-variable,expression-not-assigned,singleton-comparison

from slackoff.config import Workspace, settings


def describe_activate():
    def it_updates_workspaces(expect):
        settings.activate("Foobar")
        expect(settings.workspaces).contains(Workspace("Foobar", True, 1))


def describe_deactivate():
    def it_updates_workspaces(expect):
        settings.deactivate("Foobar")
        expect(settings.workspaces).contains(
            Workspace("Foobar", False, expect.anything)
        )



from pkg_resources import DistributionNotFound, get_distribution

try:
    __version__ = get_distribution("slackoff").version
except DistributionNotFound:
    __version__ = "(local)"

del DistributionNotFound
del get_distribution



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slackoff', 'slackoff.tests']

package_data = \
{'': ['*']}

install_requires = \
['applescript>=2021.2.9,<2022.0.0',
 'click',
 'datafiles>=1.3b1,<2.0',
 'minilog',
 'pync>=2.0.3,<3.0.0']

entry_points = \
{'console_scripts': ['slackoff = slackoff.cli:main']}

setup_kwargs = {
    'name': 'slackoff',
    'version': '0.0.10',
    'description': 'Automatically sign out of Slack workspaces on macOS.',
    'long_description': '# Overview\n\nSlackoff is a quick way to sign out of a company Slack workspace at the end of the day to improve one\'s work-life balance. It can also be used to sign out of "fun" Slack workspaces to avoid distractions during normal working hours.\n\n[![Build Status](https://img.shields.io/github/workflow/status/jacebrowning/slackoff/main)](https://github.com/jacebrowning/slackoff/actions)\n[![Coverage Status](https://img.shields.io/codecov/c/gh/jacebrowning/slackoff)](https://codecov.io/gh/jacebrowning/slackoff)\n[![Scrutinizer Code Quality](https://img.shields.io/scrutinizer/g/jacebrowning/slackoff.svg)](https://scrutinizer-ci.com/g/jacebrowning/slackoff)\n[![PyPI License](https://img.shields.io/pypi/l/slackoff.svg)](https://pypi.org/project/slackoff)\n[![PyPI Version](https://img.shields.io/pypi/v/slackoff.svg)](https://pypi.org/project/slackoff)\n[![PyPI Downloads](https://img.shields.io/pypi/dm/slackoff.svg?color=orange)](https://pypistats.org/packages/slackoff)\n\n## Setup\n\n### Requirements\n\n* macOS (for AppleScript)\n* Slack for Mac\n* Python 3.10+\n\n### Installation\n\nInstall this tool globally with [pipx](https://pipxproject.github.io/pipx/) (or pip):\n\n```sh\n$ pipx install slackoff\n```\nor add it to your [Poetry](https://python-poetry.org/docs/) project:\n\n```sh\n$ poetry add slackoff\n```\n\n## Usage\n\nAfter installation, automatically sign out of a Slack workspace:\n\n```sh\n$ slackoff My Company Workspace\n```\n\nor sign back in:\n\n```sh\n$ slackoff\n```\n\nSlackoff will remember the last workspace used and attempt to toggle appropriately.\n\n### Additional Options\n\nTo explicitly attempt to sign in or out, include the corresponding flag:\n\n```sh\n$ slackoff --signin\n$ slackoff --signout\n```\n\nView the help for more options:\n\n```sh\n$ slackoff --help\n```\n',
    'author': 'Jace Browning',
    'author_email': 'jacebrowning@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pypi.org/project/slackoff',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path

from . import script

FUNCTIONS = Path(__file__).parent / "slack.applescript"


def activate() -> bool:
    return script.call(FUNCTIONS, "activate()")


def signin(workspace: str) -> bool:
    return script.call(FUNCTIONS, f'signin("{workspace}")')


def signout(workspace: str) -> bool:
    return script.call(FUNCTIONS, f'signout("{workspace}")', show_error=False)
